/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: %W% %G% */
/*************************************************************************
 * FILE:      putScopeSetup.c
 * Author:    Claude Saunders
 * 
 * Purpose:   Program for sending scope setup data to an
 *            appropriately configured waveform record.
 *
 * Mods:      login mm/dd/yy: description
 *************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "cadef.h"
#include "ezca.h"
#include "SDDS.h"          /* SDDS library header */

#define  VERSION   "V1.0"
#define  SDDS 1
#define  NELEMENTS 2058

/* Function Prototypes */
int process_cmdline_args(int argc, char *argv[], char *scopeid, char *file);
void usage(char *name);

int main(int argc, char *argv[]) {
  char scopeid[255];            /* name of scope */
  char setupfile[255];          /* name of file for setup data */
  char writepv[255];
  char oldupdatepv[255];
  char updatepv[255];
  chid updatechid;
  unsigned char *setupdata;
  short one = 1;
  int status;                  /* status of ezca operations */
  SDDS_TABLE table;
  char *contents;              /* contents field of SDDS descrip. */
  long nrows;                  /* number of data rows in setupfile */
#if (SDDS==1)
#else
  FILE *setupfp;
#endif
  process_cmdline_args(argc, argv, scopeid, setupfile);

#if (SDDS==1)
  if (!SDDS_InitializeInput(&table, setupfile)) {
    fprintf(stderr,"%s: SDDS error: unable to initialize with file %s\n",
	    argv[0], setupfile);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  
  if (SDDS_ReadTable(&table) <= 0) {
    fprintf(stderr,"%s: SDDS error: unable to read data table from %s\n",
	    argv[0], setupfile);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  SDDS_SetColumnFlags(&table, 1);
  SDDS_SetRowFlags(&table, 1);
  
  contents = NULL;
  if (!SDDS_GetDescription(&table, NULL, &contents)) {
    fprintf(stderr,"%s: SDDS error: unable to get description from file %s\n",
	    argv[0], setupfile);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  if (contents==NULL || strcmp(contents,"scope setup data")) {
    fprintf(stderr,
	    "%s: SDDS error: file %s does not contain scope setup data\n",
	    argv[0],setupfile);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  
  if ((nrows = SDDS_CountRowsOfInterest(&table)) == -1) {
    fprintf(stderr,"%s: SDDS error: unable to get row count\n",argv[0]);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  
  if (nrows != NELEMENTS) {
    fprintf(stderr, "%s: setup data element count wrong\n",argv[0]);
    return(1);
  }
  
  if ((setupdata = SDDS_GetColumn(&table, "scopeSetup"))==NULL) {
    fprintf(stderr,"%s: SDDS error: unable to read scopeSetup column\n",
	    argv[0]);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
  if (!SDDS_Terminate(&table)) {
    fprintf(stderr,"%s: SDDS error: unable to terminate SDDS\n",argv[0]);
    SDDS_PrintErrors(stderr,(SDDS_EXIT_PrintErrors|SDDS_VERBOSE_PrintErrors));
  }
#else
  setupdata = (unsigned char *)malloc((NELEMENTS+100)*sizeof(unsigned char));
  if (setupdata == NULL) {
    fprintf(stderr,"%s: error: unable to malloc needed memory\n",argv[0]);
    return(1);
  } 
  setupfp = fopen(setupfile,"r");
  if (setupfp == NULL) {
    fprintf(stderr,"%s: error: unable to open %s for reading\n",argv[0],
	    setupfile);
    return(1);
  }
  if (fread(setupdata, sizeof(unsigned char), NELEMENTS, setupfp) != 
      NELEMENTS) {
    fprintf(stderr,"%s: error: unable to read data from %s\n",argv[0],
	    setupfile);
    return(1);
  }
  fclose(setupfp);
#endif
  /* Form proper process variable name */
  strcpy(writepv,scopeid);
  strcat(writepv,":writeSetupWF");

  strcpy(oldupdatepv,scopeid);
  strcat(oldupdatepv,":update1SEQ");
  strcpy(updatepv,scopeid);
  strcat(updatepv,":update1SQ");

  ezcaSetTimeout(0.3);
  ezcaSetRetryCount(200);

  /* Write scope setup data */
  status = ezcaPut(writepv, ezcaByte, NELEMENTS, setupdata);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope\n",argv[0]);
    ca_task_exit();
    return(1);
  }

  sleep(2);
  status = ca_search(updatepv, &updatechid);
  if (status != ECA_NORMAL) {
    fprintf(stderr,"%s: unable to write scope update\n",argv[0]);
    ca_task_exit();
    return(1);
  }
  status = ca_pend_io(5.0);
  if (status != ECA_NORMAL) {
    status = ca_search(oldupdatepv, &updatechid);
    if (status != ECA_NORMAL) {
      fprintf(stderr,"%s: unable to write scope update\n",argv[0]);
      ca_task_exit();
      return(1);
    }
    status = ca_pend_io(5.0);
    if (status != ECA_NORMAL) {
      fprintf(stderr,"%s: unable to write scope update\n",argv[0]);
      ca_task_exit();
      return(1);
    }
  }
  status = ca_put(DBR_SHORT, updatechid, &one);
  if (status != ECA_NORMAL) {
    fprintf(stderr,"%s: unable to write scope update\n",argv[0]);
    ca_task_exit();
    return(1);
  }
  status = ca_pend_io(5.0);
  if (status != ECA_NORMAL) {
    fprintf(stderr,"%s: unable to write scope update\n",argv[0]);
    ca_task_exit();
    return(1);
  }
  ca_task_exit();
  return(0);
}
/*************************************************************************
 * FUNCTION : process_cmdline_args()
 * PURPOSE  : Get command line arguments and set flags appropriately.
 * ARGS in  : argc, argv - standard unix
 * ARGS out : scopeid - name of scope
 *            setupfile - name of file to dump scope setup into
 * GLOBAL   : nothing
 * RETURNS  : 0
 ************************************************************************/  
int
process_cmdline_args(int argc, char *argv[], char *scopeid, char *setupfile)
{
  int cmderror = 0;

  *scopeid = '\0';
  *setupfile = '\0';

  /* Process command line options */
  if (argc != 3)
    cmderror++;
  else {
    strcpy(scopeid,argv[1]);
    strcpy(setupfile,argv[2]);
    }

  if (cmderror) {
    usage(argv[0]);
    exit(1);
  }
  return(0);
}

/*************************************************************************
 * FUNCTION : usage()
 * PURPOSE  : Describes command line options available to client.
 * ARGS in  : char *name - name of executable program
 * ARGS out : none
 * GLOBAL   : nothing
 * RETURNS  : none
 ************************************************************************/  
void usage(char *name)
{
  fprintf(stderr,"Put Scope Setup Utility %s\n",VERSION);
  fprintf(stderr,"----------------------------\n");
  fprintf(stderr,"usage: %s <scope-id> <setup-filename>\n",name);
  fprintf(stderr,"purpose:\tWrites the scope setup in <setup-filename>\n");
  fprintf(stderr,"\t\tto the scope <scope-id>.\n");
}

