/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* SCCS ID: %W% %G% */
/*************************************************************************
 * FILE:      sendScopeCommand.c
 * Author:    Claude Saunders
 * 
 * Purpose:   Program for sending an arbitrary GPIB command
 *            to an instrument. This is a tricky business given
 *            EPICS and the GPIB support library, so use it with
 *            caution. In particular, it is assumed that ALL queries
 *            have '?' somewhere in the query, and ALL commands do not.
 *
 * Mods:      login mm/dd/yy: description
 *
 *            01/12/99  nda  added support to use a waveform record so commands
 *                           and responses could be up to 255 characters.
 *
 *
 *************************************************************************/

#include <stdio.h>
#include <string.h>
#include "cadef.h"
#include "ezca.h"
#include "SDDS.h"         /* SDDS library header */

#define  VERSION   "V1.3"


#define  WF_ARRAY_SIZE 255     /* size of character array defined in database */
                               /* MAX_STRING_SIZE = 40  from epicsTypes.h */


/* Function Prototypes */
int process_cmdline_args(int argc, char *argv[], char *scopeid, char *cmd);
void usage(char *name);

int main(int argc, char *argv[]) {
  char scopeid[255];            /* name of scope */
  char cmd[WF_ARRAY_SIZE];      /* command/query string sent to instrument */
  char resp[WF_ARRAY_SIZE];     /* response to query */
  char writepv[255];
  char readpv[255];
  char procpv[255];
  short one = 1;
  int status;                  /* status of ezca operations */
  int nelem;                   /* number of elements of PV  */
  int maxSize; 

  process_cmdline_args(argc, argv, scopeid, cmd);
  
  /* Form proper process variable name */
  strcpy(writepv,scopeid);
  strcat(writepv,":sendCmdSO");

  strcpy(readpv,scopeid);
  strcat(readpv,":sendCmdSI");

  strcpy(procpv,scopeid);
  strcat(procpv,":sendCmdSI.PROC");

  ezcaSetTimeout(0.1);
  ezcaSetRetryCount(100);

  /* Find out if sendCmdSI is a string record or a waveform record */
  status = ezcaGetNelem(readpv, &nelem);
  if (status) {
    fprintf(stderr, "%s: Unable to find desired scope\n",argv[0]);
    ca_task_exit();
    return(1);
  }
  
  /* If nelem = 1, record type is a string, otherwise a waveform */
  if(nelem > 1) 
     maxSize = WF_ARRAY_SIZE;
  else 
     maxSize = MAX_STRING_SIZE;
  
 
  /* Note: we are limited here by the stringout val field length */

  if (strlen(cmd) > maxSize) {
    fprintf(stderr,"%s: Command/Query length limited to %d chars\n",
            argv[0], maxSize);
    ca_task_exit();
    return(1);
  }

  /* Write scope command string , dependent on PV type */
  if(nelem > 1) {
      status = ezcaPut(writepv, ezcaByte, strlen(cmd), cmd);
      if (status) {
        fprintf(stderr, "%s: Unable to find desired scope\n",argv[0]);
	ca_task_exit();
        return(1);
      }

      if (strchr(cmd, '?')) {
        /* Process the record to get the query response */
        status = ezcaPut(procpv, ezcaShort, 1, &one);
        if (status) {
          fprintf(stderr, "%s: Unable to retrieve response\n",argv[0]);
	  ca_task_exit();
          return(1);
        }
        /* Read the query response */
        status = ezcaGet(readpv, ezcaByte, WF_ARRAY_SIZE, &resp);
        if (status) {
          fprintf(stderr, "%s: Unable to retrieve response\n",argv[0]);
	  ca_task_exit();
          return(1);
        }

        resp[WF_ARRAY_SIZE - 1] = '\0';  /* protect against overflow */
        fprintf(stdout,"%s",resp);
      }
  }
  else {
      status = ezcaPut(writepv, ezcaString, 1, cmd);
      if (status) {
        fprintf(stderr, "%s: Unable to find desired scope\n",argv[0]);
	ca_task_exit();
	return(1);
      }

      if (strchr(cmd, '?')) {
        /* Process the record to get the query response */
        status = ezcaPut(procpv, ezcaShort, 1, &one);
        if (status) {
          fprintf(stderr, "%s: Unable to retrieve response\n",argv[0]);
	  ca_task_exit();
          return(1);
        }
        /* Read the query response */
        status = ezcaGet(readpv, ezcaString, 1, &resp);
        if (status) {
          fprintf(stderr, "%s: Unable to retrieve response\n",argv[0]);
	  ca_task_exit();
          return(1);
        }
        resp[MAX_STRING_SIZE] = '\0';
        fprintf(stdout,"%s",resp);
      }
  }
  ca_task_exit();

  return(0);
}

/*************************************************************************
 * FUNCTION : process_cmdline_args()
 * PURPOSE  : Get command line arguments and set flags appropriately.
 * ARGS in  : argc, argv - standard unix
 * ARGS out : scopeid - name of scope
 *            cmd - command/query string to be sent to GPIB instrument
 * GLOBAL   : nothing
 * RETURNS  : 0
 ************************************************************************/  
int
process_cmdline_args(int argc, char *argv[], char *scopeid, char *cmd)
{
  int cmderror = 0;

  *scopeid = '\0';
  *cmd = '\0';

  /* Process command line options */
  if (argc != 3)
    cmderror++;
  else {
    strcpy(scopeid,argv[1]);
    strcpy(cmd,argv[2]);
    }

  if (cmderror) {
    usage(argv[0]);
    exit(1);
  }
  return(0);
}

/*************************************************************************
 * FUNCTION : usage()
 * PURPOSE  : Describes command line options available to client.
 * ARGS in  : char *name - name of executable program
 * ARGS out : none
 * GLOBAL   : nothing
 * RETURNS  : none
 ************************************************************************/  
void usage(char *name)
{
  fprintf(stderr,"Send Scope Command Utility %s\n",VERSION);
  fprintf(stderr,"-------------------------------\n");
  fprintf(stderr,"usage: %s <scope-id> <GPIB-command>\n",name);
  fprintf(stderr,"purpose:\tWrites <GPIB-command> to scope <scope-id>.\n");
  fprintf(stderr,"\t\tAny response is written to stdout. It is assumed\n");
  fprintf(stderr,"\t\tthat all queries contain the ? character.\n");
}

